import os

from blabel import LabelWriter

from .settings import BASE_DIR


def generate_order_labels(orders, temp_dir):
    records = []
    for order in orders:
        record = {
            "barcode": order.id,
            "full_name": order.full_name,
            "total": order.total,
            "items_count": order.items_count,
        }
        records.append(record)
    label_writer = LabelWriter(
        os.path.join(BASE_DIR, "order_label.html"),
        default_stylesheets=(os.path.join(BASE_DIR, "order_label_style.css"),),
    )
    label_writer.write_labels(records, target=os.path.join(temp_dir, "labels.pdf"))
    return os.path.join(temp_dir, "labels.pdf")


def generate_address_labels(orders, temp_dir):
    records = []
    for order in orders:
        record = {
            "sender": order.form.sender,
            "full_name": order.shipping_full_name,
            "second_address_row": order.second_address_row,
            "street": order.street,
            "housenumber": order.housenumber,
            "plz": order.plz,
            "place": order.place,
        }
        records.append(record)
    label_writer = LabelWriter(
        os.path.join(BASE_DIR, "address_label.html"),
        default_stylesheets=(os.path.join(BASE_DIR, "address_label_style.css"),),
    )
    label_writer.write_labels(records, target=os.path.join(temp_dir, "labels.pdf"))
    return os.path.join(temp_dir, "labels.pdf")



from django.db.models.aggregates import Sum

import django_filters
from material import Layout, Row

from aleksis.apps.order.models import Order
from aleksis.core.util.core_helpers import queryset_rules_filter


class OrderFilter(django_filters.FilterSet):
    full_name = django_filters.CharFilter(lookup_expr="icontains")
    items_count = django_filters.NumericRangeFilter(method="items_count_filter")

    def items_count_filter(self, queryset, name, value):
        return queryset.annotate(items_count_db=Sum("items__count")).filter(
            items_count_db__gte=value.start, items_count_db__lte=value.stop
        )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.form.layout = Layout(
            Row("form"),
            Row("full_name", "email"),
            Row("confirmed", "paid", "sent"),
            Row("processing_option", "items_count"),
        )

    @property
    def qs(self):
        parent = super().qs.order_by("full_name")
        user = getattr(self.request, "user", None)

        return queryset_rules_filter(user, parent, "order.manage_order")

    class Meta:
        model = Order
        fields = ["form", "full_name", "email", "confirmed", "paid", "sent", "processing_option"]



# Generated by Django 3.1.3 on 2020-12-18 11:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('order', '0003_auto_20201218_1153'),
    ]

    operations = [
        migrations.AddField(
            model_name='order',
            name='notes',
            field=models.TextField(blank=True, verbose_name='Notizen'),
        ),
    ]



# Generated by Django 3.1.3 on 2020-12-15 20:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('order', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='order',
            name='housenumber',
            field=models.CharField(blank=True, max_length=255, verbose_name='Housenumber'),
        ),
        migrations.AlterField(
            model_name='order',
            name='street',
            field=models.CharField(blank=True, max_length=255, verbose_name='Straße'),
        ),
    ]



import rules

from aleksis.core.util.predicates import (
    has_any_object,
    has_global_perm,
    has_object_perm,
    has_person,
)

from .models import OrderForm
from .util.predicates import has_order_perm

manage_orders_of_form_predicate = has_person & (
    has_global_perm("order.manage_orders") | has_object_perm("order.manage_orders_of_form")
)
rules.add_perm("order.manage_orders_of_form", manage_orders_of_form_predicate)

view_orders_predicate = has_person & (
    has_global_perm("order.manage.orders")
    | has_any_object("order.manage_orders_of_form", OrderForm)
)
rules.add_perm("order.view_orders", view_orders_predicate)

manage_order_predicate = has_person & (has_global_perm("order.manage_orders") | has_order_perm)
rules.add_perm("order.manage_order", manage_order_predicate)

show_menu_predicate = view_orders_predicate
rules.add_perm("order.show_menu", show_menu_predicate)



# Generated by Django 3.1.5 on 2021-01-11 13:32

import ckeditor.fields
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('order', '0006_orderform_access_code'),
    ]

    operations = [
        migrations.AddField(
            model_name='orderform',
            name='closed',
            field=models.BooleanField(default=False, verbose_name='Form closed for orders'),
        ),
    ]



from django.contrib.auth.models import User

from rules import predicate

from aleksis.core.util.predicates import has_object_perm

from ..models import Order


@predicate
def has_order_perm(user: User, obj: Order) -> bool:
    return has_object_perm("order.manage_orders_of_form")(user, obj.form)



from tempfile import TemporaryDirectory
from typing import Any, Dict

from django.contrib import messages
from django.http import HttpRequest, HttpResponse
from django.http.response import FileResponse, HttpResponseNotFound
from django.shortcuts import get_object_or_404, redirect, render
from django.urls import reverse_lazy
from django.utils.translation import gettext as _
from django.views.generic.base import View
from django.views.generic.detail import DetailView

from django_filters.views import FilterView
from django_tables2 import SingleTableMixin
from rules.contrib.views import PermissionRequiredMixin

from aleksis.apps.order.filters import OrderFilter
from aleksis.apps.order.pdf import generate_address_labels, generate_order_labels
from aleksis.apps.order.tables import OrderTable
from aleksis.core.mixins import AdvancedDeleteView, AdvancedEditView
from aleksis.core.util.core_helpers import has_person, queryset_rules_filter
from aleksis.core.util.pdf import render_pdf

from .forms import (
    AccessForm,
    NotesForm,
    OrderActionForm,
    OrderFormForm,
    OrderItemFormSet,
    ProcessingOptionForm,
    ShippingAddressForm,
)
from .models import Order, OrderForm, OrderItem


class OrderFormView(DetailView):
    model = OrderForm
    template_name = "order/form.html"

    def get(self, request, *args, **kwargs):
        self.object = self.get_object()
        context = self.get_context_data(object=self.object)
        form_access_granted_key = f"order_access_{self.object.pk}"

        if self.object.closed:
            return redirect("index")

        access_granted = (
            has_person(self.request)
            or request.session.get("order")
            or request.session.get(form_access_granted_key)
            or not self.object.access_code.strip()
        )
        if access_granted:
            order = self.request.session.get("order")
            try:
                order = Order.objects.get(pk=order)
            except Order.DoesNotExist:
                order = None

            form = OrderFormForm(self.request.POST or None, instance=order)
            formset = OrderItemFormSet(
                self.request.POST or None,
                queryset=self.object.available_items.all(),
                prefix="order_item",
            )

            # Load order from session
            if order:
                counts_by_pk = {item.item.pk: item.count for item in order.items.all()}
                for item_form in formset:
                    item_form.fields["count"].initial = counts_by_pk.get(item_form.instance.pk, 0)

            context["form"] = form
            context["formset"] = formset

        if not access_granted:
            # Show access code form
            access_form = AccessForm(request.POST or None)
            context["access_form"] = access_form
            if request.method == "POST" and access_form.is_valid():
                if (
                    self.object.access_code.lower().strip()
                    == access_form.cleaned_data["access_code"]
                ):
                    request.session[form_access_granted_key] = True
                    return redirect("order_form", self.object.pk)
                else:
                    messages.error(request, _("Please enter the correct access code."))

            return render(request, "order/access.html", context)

        person = None
        if has_person(self.request):
            person = self.request.user.person
            form.fields["full_name"].disabled = True
            form.fields["full_name"].initial = person.addressing_name
            if person.email:
                form.fields["email"].disabled = True
                form.fields["email"].initial = person.email

        if self.request.method == "POST":
            items = []
            if form.is_valid():
                full_name = person.addressing_name if person else form.cleaned_data["full_name"]
                email = (
                    person.email if getattr(person, "email", None) else form.cleaned_data["email"]
                )

                # Get items
                if formset.is_valid():
                    total_count_of_items = 0
                    for item_form in formset:
                        # Calculate count
                        count = item_form.cleaned_data["count"]
                        total_count_of_items += count

                        if count > 0:
                            order_item = None

                            # Check for existing order item
                            if order:
                                try:
                                    order_item = OrderItem.objects.get(
                                        item=item_form.instance, order=order
                                    )
                                except OrderItem.DoesNotExist:
                                    pass

                            # New order item
                            if not order_item:
                                order_item = OrderItem(item=item_form.instance, count=count)

                            order_item.new_count = count
                            items.append(order_item)

                    # Check if there is at least one item
                    if total_count_of_items <= 0:
                        messages.error(self.request, _("You must order at least one item."))
                        return self.render_to_response(context)

                    # Get or create order
                    if not order:
                        order = Order.objects.create(
                            form=self.object, full_name=full_name, email=email
                        )
                    else:
                        order.full_name = full_name
                        order.email = email
                        order.save()

                    # Save order items
                    for item in items:
                        item.order = order
                        item.count = item.new_count
                        item.save()

                    # Set session and redirect
                    self.request.session["order"] = order.pk
                    return redirect("order_form_2", self.object.pk)
        # TODO DON'T GET FULL NAME AND EMAIL FROM FORM IF USER IS LOGGED IN

        return self.render_to_response(context)

    def post(self, *args, **kwargs):
        return self.get(*args, **kwargs)


class OrderForm2View(DetailView):
    model = OrderForm
    template_name = "order/form2.html"

    def get(self, request: HttpRequest, *args: Any, **kwargs: Any) -> HttpResponse:
        self.object = self.get_object()
        context = self.get_context_data(object=self.object)
        order = self.request.session.get("order")
        try:
            self.order = Order.objects.get(pk=order)
        except Order.DoesNotExist:
            return redirect("order_form", self.object.pk)
        if self.order.form != self.get_object():
            return redirect("order_form_2", self.order.form.pk)

        if hasattr(self, "order"):
            context["order"] = self.order

            processing_form = ProcessingOptionForm(
                self.request.POST or None,
                initial={"processing_option": self.order.processing_option},
            )
            context["processing_form"] = processing_form

            processing_options = self.order.form.available_processing_options.all()
            for processing_option in processing_options:
                processing_option.price = processing_option.get_price(self.order.items_count)

            context["processing_options"] = processing_options

            shipping_address_form = ShippingAddressForm(
                self.request.POST or None,
                initial={
                    "full_name": self.order.shipping_full_name,
                    "second_address_row": self.order.second_address_row,
                    "street": self.order.street,
                    "housenumber": self.order.housenumber,
                    "plz": self.order.plz,
                    "place": self.order.place,
                },
            )
            context["shipping_address_form"] = shipping_address_form

            if self.request.method == "POST":
                if processing_form.is_valid():
                    processing_option = processing_form.cleaned_data["processing_option"]

                    if not processing_option.address_necessary or shipping_address_form.is_valid():
                        # WITH SHIPPING ADDRESS
                        self.order.processing_option = processing_option

                        if processing_option.address_necessary:
                            self.order.shipping_full_name = shipping_address_form.cleaned_data[
                                "full_name"
                            ]
                            self.order.second_address_row = shipping_address_form.cleaned_data[
                                "second_address_row"
                            ]
                            self.order.street = shipping_address_form.cleaned_data["street"]
                            self.order.housenumber = shipping_address_form.cleaned_data[
                                "housenumber"
                            ]
                            self.order.plz = shipping_address_form.cleaned_data["plz"]
                            self.order.place = shipping_address_form.cleaned_data["place"]

                        self.order.save()

                        return redirect("order_form_3", self.order.form.pk)
                else:
                    messages.error(request, _("Please select a processing option."))
        return self.render_to_response(context)

    def post(self, *args, **kwargs):
        return self.get(*args, **kwargs)


class OrderForm3View(DetailView):
    model = OrderForm
    template_name = "order/form3.html"

    def get(self, request: HttpRequest, *args: Any, **kwargs: Any) -> HttpResponse:
        self.object = self.get_object()
        context = self.get_context_data(object=self.object)
        order = self.request.session.get("order")
        try:
            self.order = Order.objects.get(pk=order)
        except Order.DoesNotExist:
            return redirect("order_form", self.object.pk)
        if self.order.form != self.get_object():
            return redirect("order_form_3", self.order.form.pk)

        context["order"] = self.order
        notes_form = NotesForm(request.POST or None, initial={"notes": self.order.notes})
        context["notes_form"] = notes_form
        if notes_form.is_valid():
            self.order.notes = notes_form.cleaned_data["notes"]
            self.order.save()
            return redirect("order_finished", self.order.form.pk)

        return self.render_to_response(context)

    def post(self, *args, **kwargs):
        return self.get(*args, **kwargs)


class OrderFinishedView(DetailView):
    model = OrderForm
    template_name = "order/finished.html"

    def get(self, request: HttpRequest, *args: Any, **kwargs: Any) -> HttpResponse:
        order = self.request.session.get("order")
        try:
            self.order = Order.objects.get(pk=order)
        except Order.DoesNotExist:
            return redirect("order_form", self.get_object().pk)
        r = super().get(request, *args, **kwargs)
        if self.order.form != self.get_object():
            return redirect("order_finished", self.order.form.pk)
        return r

    def get_context_data(self, **kwargs: Any) -> Dict[str, Any]:
        context = super().get_context_data(**kwargs)
        if hasattr(self, "order"):
            context["order"] = self.order
            self.order.submitted = True
            self.order.save()
            self.order.send_overview(self.request)
            form_access_granted_key = f"order_access_{self.order.form.pk}"

            del self.request.session["order"]
            if form_access_granted_key in self.request.session:
                del self.request.session[form_access_granted_key]
        return context


class OrderConfirmView(View):
    def get(self, request: HttpRequest, key, **kwargs: Any) -> HttpResponse:
        try:
            self.order = Order.objects.get(confirm_key=key)
        except Order.DoesNotExist:
            return HttpResponseNotFound()
        if not self.order.confirmed:
            self.order.confirm(request)
        return render(request, "order/confirmed.html", context={"order": self.order})


class OrderDetailView(PermissionRequiredMixin, DetailView):
    model = Order
    permission_required = "order.manage_order"
    template_name = "order/detail.html"


class OrderEditView(PermissionRequiredMixin, AdvancedEditView):
    model = Order
    fields = (
        "full_name",
        "email",
        "notes",
        "submitted",
        "confirmed",
        "paid",
        "sent",
        "processing_option",
        "shipping_full_name",
        "second_address_row",
        "street",
        "housenumber",
        "plz",
        "place",
    )
    permission_required = "order.manage_order"
    template_name = "order/edit.html"
    success_message = _("The order has been changed successfully.")
    success_url = reverse_lazy("list_orders")

    def post(self, request, *args, **kwargs):
        r = super().post(request, *args, **kwargs)

        order = self.get_object()

        if request.POST.get("mark-as-paid"):
            order.paid = True
            order.save()

        elif request.POST.get("mark-as-paid-email"):
            order.send_pay_confirmation()

        return r


class OrderDeleteView(PermissionRequiredMixin, AdvancedDeleteView):
    model = Order
    permission_required = "order.manage_order"
    template_name = "core/pages/delete.html"
    success_message = _("The order has been deleted successfully.")
    success_url = reverse_lazy("list_orders")


class OrderListView(PermissionRequiredMixin, SingleTableMixin, FilterView):
    table_class = OrderTable
    model = Order
    template_name = "order/list.html"
    permission_required = "order.view_orders"
    table_pagination = {"per_page": 50}

    filterset_class = OrderFilter

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["table"].request = self.request
        context["forms"] = queryset_rules_filter(
            self.request.user, OrderForm.objects.all(), "order.manage_orders_of_form"
        )
        self.action_form = OrderActionForm(
            self.request, self.request.POST or None, queryset=self.get_queryset()
        )
        context["action_form"] = self.action_form
        return context

    def post(self, request, *args, **kwargs):
        r = self.get(request, *args, **kwargs)

        if "action" in self.request.POST and self.action_form.is_valid():
            self.action_form.execute()

        elif request.POST.get("all-action"):
            if request.POST.get("print"):
                return render_pdf(request, "order/list_print.html", self.get_context_data())
            elif request.POST.get("packing-lists"):
                return render_pdf(request, "order/packing_lists.html", self.get_context_data())

        # Action for a single object
        elif request.POST.get("single-action-pk"):
            order = get_object_or_404(Order, pk=request.POST.get("single-action-pk"))

            if request.POST.get("mark-as-paid"):
                order.paid = True
                order.save()

            elif request.POST.get("mark-as-paid-email"):
                order.send_pay_confirmation()

            elif request.POST.get("mark-as-sent"):
                order.sent = True
                order.save()

        # Actions for all objects
        elif request.POST.get("all-action"):
            if request.POST.get("mark-as-paid"):
                self.object_list.update(paid=True)

            elif request.POST.get("mark-as-paid-email"):
                self.object_list.update(paid=True)
                for order in self.object_list:
                    order.send_pay_confirmation()
            elif request.POST.get("mark-as-sent"):
                self.object_list.update(sent=True)

            elif request.POST.get("labels"):
                with TemporaryDirectory() as temp_dir:
                    filename = generate_order_labels(self.object_list, temp_dir)
                    f = open(filename, "rb")
                    return FileResponse(f, filename="labels.pdf")
            elif request.POST.get("address-labels"):
                with TemporaryDirectory() as temp_dir:
                    filename = generate_address_labels(self.object_list, temp_dir)
                    f = open(filename, "rb")
                    return FileResponse(f, filename="labels.pdf")
        return r



# Generated by Django 3.1.3 on 2020-12-18 10:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('order', '0002_auto_20201215_2136'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='order',
            name='in_progress',
        ),
        migrations.AddField(
            model_name='order',
            name='submitted',
            field=models.BooleanField(default=False, verbose_name='Submitted'),
        ),
    ]



from django import forms
from django.core.exceptions import ValidationError
from django.core.validators import MinValueValidator
from django.utils.translation import gettext as _

from material import Layout, Row, Span2

from aleksis.core.forms import ActionForm

from .models import Item, Order, ProcessingOption


class OrderFormForm(forms.ModelForm):
    class Meta:
        model = Order
        fields = ["full_name", "email"]


class OrderItemForm(forms.ModelForm):
    count = forms.IntegerField(label=_("Count"), initial=0, validators=[MinValueValidator(0)])

    def clean_count(self):
        count = self.cleaned_data["count"]

        if count > self.instance.max_count:
            raise ValidationError(
                _(f"You can only order {self.instance.max_count} pieces of this item.")
            )

        return count

    class Meta:
        model = Item
        fields = ["count"]


OrderItemFormSet = forms.modelformset_factory(Item, form=OrderItemForm, max_num=0, extra=0)


class ProcessingOptionForm(forms.Form):
    processing_option = forms.ModelChoiceField(ProcessingOption.objects.all())


class ShippingAddressForm(forms.Form):
    layout = Layout(
        Row("full_name"),
        Row("second_address_row"),
        Row(Span2("street"), "housenumber"),
        Row("plz", Span2("place")),
    )
    full_name = forms.CharField(label=_("First and last name"))
    second_address_row = forms.CharField(label=_("Second address row"), required=False)
    street = forms.CharField(label=_("Street"))
    housenumber = forms.CharField(label=_("Housenumber"))
    plz = forms.CharField(label=_("PLZ"), max_length=5)
    place = forms.CharField(label=_("Place"))


class NotesForm(forms.Form):
    notes = forms.CharField(widget=forms.Textarea, label=_("Notes to your order"), required=False)


class AccessForm(forms.Form):
    access_code = forms.CharField(
        label=_("Access code"), widget=forms.TextInput(attrs={"autofocus": "autofocus"})
    )

    def clean_access_code(self):
        return self.cleaned_data.get("access_code", "").lower().strip()


def send_confirmation_reminder(modeladmin, request, queryset):
    qs = queryset.filter(submitted=True, confirmed=False)
    for order in qs:
        order.send_confirmation_reminder(request)


send_confirmation_reminder.short_description = _("Send confirmation reminder")


def send_pay_reminder(modeladmin, request, queryset):
    qs = queryset.filter(submitted=True, confirmed=True, paid=False)
    for order in qs:
        order.send_pay_reminder(request)


send_pay_reminder.short_description = _("Send pay reminder")


class OrderActionForm(ActionForm):
    actions = [send_confirmation_reminder, send_pay_reminder]



from django.template.loader import render_to_string

from django_tables2 import A, Column, LinkColumn, tables

from aleksis.apps.order.models import Order
from aleksis.core.util.tables import SelectColumn


class OrderTable(tables.Table):
    selected = SelectColumn()
    full_name = LinkColumn("show_order", args=[A("pk")])
    actions = Column(accessor="pk")

    def render_actions(self, value):
        return render_to_string("order/actions.html", {"pk": value}, self.request)

    class Meta:
        model = Order
        order_by = "-created"
        fields = (
            "id",
            "form",
            "created",
            "full_name",
            "email",
            "notes",
            "confirmed",
            "paid",
            "sent",
            "processing_option",
            "processing_price",
            "items_count",
            "total",
        )
        sequence = ("selected", "...", "actions")



# Generated by Django 3.1.4 on 2020-12-15 16:13

import ckeditor.fields
import django.contrib.sites.managers
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import model_utils.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
    ]

    operations = [
        migrations.CreateModel(
            name='Item',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('extended_data', models.JSONField(default=dict, editable=False)),
                ('short_name', models.CharField(max_length=255, unique=True, verbose_name='Kurzname')),
                ('name', models.CharField(max_length=255, verbose_name='Name')),
                ('notice', ckeditor.fields.RichTextField(blank=True, help_text='Shown below the item', verbose_name='Notice')),
                ('price', models.FloatField(validators=[django.core.validators.MinValueValidator(0.0)])),
                ('max_count', models.IntegerField(default=0, validators=[django.core.validators.MinValueValidator(0)], verbose_name='Maximum count of items')),
                ('site', models.ForeignKey(default=1, editable=False, on_delete=django.db.models.deletion.CASCADE, to='sites.site')),
            ],
            options={
                'verbose_name': 'Item',
                'verbose_name_plural': 'Items',
            },
            managers=[
                ('objects', django.contrib.sites.managers.CurrentSiteManager()),
            ],
        ),
        migrations.CreateModel(
            name='Order',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('extended_data', models.JSONField(default=dict, editable=False)),
                ('full_name', models.CharField(max_length=255, verbose_name='First and last name')),
                ('email', models.EmailField(max_length=254, verbose_name='E-Mail-Adresse')),
                ('in_progress', models.BooleanField(default=True, verbose_name='In progress')),
                ('confirmed', models.BooleanField(default=False, verbose_name='Confirmed')),
                ('confirm_key', models.TextField(blank=True, verbose_name='Confirm key')),
                ('paid', models.BooleanField(default=False, verbose_name='Paid')),
                ('sent', models.BooleanField(default=False, verbose_name='Versandt')),
                ('shipping_full_name', models.CharField(blank=True, max_length=255, verbose_name='First and last name')),
                ('second_address_row', models.CharField(blank=True, max_length=255, verbose_name='Second address row')),
                ('street', models.CharField(blank=True, max_length=255, verbose_name='Street and housenumber')),
                ('plz', models.CharField(blank=True, max_length=255, verbose_name='PLZ')),
                ('place', models.CharField(blank=True, max_length=255, verbose_name='Ort')),
            ],
            options={
                'verbose_name': 'Reihenfolge',
                'verbose_name_plural': 'Orders',
            },
            managers=[
                ('objects', django.contrib.sites.managers.CurrentSiteManager()),
            ],
        ),
        migrations.CreateModel(
            name='ProcessingOption',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('extended_data', models.JSONField(default=dict, editable=False)),
                ('short_name', models.CharField(max_length=255, unique=True, verbose_name='Kurzname')),
                ('name', models.CharField(max_length=255, verbose_name='Name')),
                ('notice', ckeditor.fields.RichTextField(blank=True, help_text='Shown below the option', verbose_name='Notice')),
                ('email_notice', ckeditor.fields.RichTextField(blank=True, help_text='Shown in the confirmation email', verbose_name='Email notice')),
                ('address_necessary', models.BooleanField(default=False, verbose_name='Adress necessary')),
                ('site', models.ForeignKey(default=1, editable=False, on_delete=django.db.models.deletion.CASCADE, to='sites.site')),
            ],
            options={
                'abstract': False,
            },
            managers=[
                ('objects', django.contrib.sites.managers.CurrentSiteManager()),
            ],
        ),
        migrations.CreateModel(
            name='ProcessingOptionPrice',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('extended_data', models.JSONField(default=dict, editable=False)),
                ('min_count', models.IntegerField(validators=[django.core.validators.MinValueValidator(0)], verbose_name='Min count of items')),
                ('max_count', models.IntegerField(validators=[django.core.validators.MinValueValidator(0)], verbose_name='Max count of items')),
                ('price', models.FloatField(validators=[django.core.validators.MinValueValidator(0.0)])),
                ('option', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='prices', to='order.processingoption', verbose_name='Processing option')),
                ('site', models.ForeignKey(default=1, editable=False, on_delete=django.db.models.deletion.CASCADE, to='sites.site')),
            ],
            options={
                'abstract': False,
            },
            managers=[
                ('objects', django.contrib.sites.managers.CurrentSiteManager()),
            ],
        ),
        migrations.CreateModel(
            name='OrderItem',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('extended_data', models.JSONField(default=dict, editable=False)),
                ('count', models.IntegerField(verbose_name='Anzahl')),
                ('item', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='order.item', verbose_name='Item')),
                ('order', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='items', to='order.order', verbose_name='Reihenfolge')),
                ('site', models.ForeignKey(default=1, editable=False, on_delete=django.db.models.deletion.CASCADE, to='sites.site')),
            ],
            options={
                'abstract': False,
            },
            managers=[
                ('objects', django.contrib.sites.managers.CurrentSiteManager()),
            ],
        ),
        migrations.CreateModel(
            name='OrderForm',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('extended_data', models.JSONField(default=dict, editable=False)),
                ('title', models.CharField(max_length=255, verbose_name='Form title')),
                ('help_text', ckeditor.fields.RichTextField(blank=True, help_text="Shown in the form's footer", verbose_name='Help text')),
                ('from_email', models.EmailField(max_length=254, verbose_name='From email address')),
                ('from_name', models.CharField(max_length=255, verbose_name='From name')),
                ('available_items', models.ManyToManyField(related_name='forms', to='order.Item', verbose_name='Available items')),
                ('available_processing_options', models.ManyToManyField(related_name='forms', to='order.ProcessingOption', verbose_name='Available processing options')),
                ('site', models.ForeignKey(default=1, editable=False, on_delete=django.db.models.deletion.CASCADE, to='sites.site')),
            ],
            options={
                'verbose_name': 'Order form',
                'verbose_name_plural': 'Order forms',
            },
            managers=[
                ('objects', django.contrib.sites.managers.CurrentSiteManager()),
            ],
        ),
        migrations.AddField(
            model_name='order',
            name='form',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='orders', to='order.orderform', verbose_name='Order form'),
        ),
        migrations.AddField(
            model_name='order',
            name='processing_option',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='order.processingoption', verbose_name='Selected processing option'),
        ),
        migrations.AddField(
            model_name='order',
            name='site',
            field=models.ForeignKey(default=1, editable=False, on_delete=django.db.models.deletion.CASCADE, to='sites.site'),
        ),
    ]



from django.urls import path

from . import views

urlpatterns = [
    path("list/", views.OrderListView.as_view(), name="list_orders"),
    path("list/<int:pk>/", views.OrderDetailView.as_view(), name="show_order"),
    path("list/<int:pk>/edit/", views.OrderEditView.as_view(), name="edit_order"),
    path("list/<int:pk>/delete/", views.OrderDeleteView.as_view(), name="delete_order"),
    path("<int:pk>/", views.OrderFormView.as_view(), name="order_form"),
    path("<int:pk>/2/", views.OrderForm2View.as_view(), name="order_form_2"),
    path("<int:pk>/3/", views.OrderForm3View.as_view(), name="order_form_3"),
    path("<int:pk>/finished/", views.OrderFinishedView.as_view(), name="order_finished"),
    path("confirm/<str:key>/", views.OrderConfirmView.as_view(), name="confirm_order"),
]
